from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import InlineResponse200_65, InlineResponse200_66


class DisburseUploadDocumentsService(BaseService):

    @cast_models
    def upload_documents_to_payout(
        self,
        payout: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        file: str,
        idempotency: str = None,
    ) -> InlineResponse200_65:
        """Upload one supporting document file or multiple document files to a specific payout. These documents include ID scans, company registrations, and all other documents that support the validity of the payout. A file can be uploaded only when the payout is in a **Created**, **Hold**, or **Pending** status.

        :param payout: ID of the payout. String starting with **payout_**.
        :type payout: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **multipart/form-data**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param file: Use the `-F` or `--form` parameter to define the path to the document file on your local machine. Supported file types include '*.pdf', '*.doc', '*.docx', '.*csv', '*.jpeg', '*.jpg', '*.gif', '*.png'. Format of input must follow the `multipart/form-data` protocol. Use the `files` parameter to upload multiple files.<BR>**Note**<BR> * Maximum number of files that can be uploaded in one request is 10.<BR>* Maximum size of files that can be uploaded in one request is 20 MB.<BR>* Document file name should be up to 200 characters long.
        :type file: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The following parameters are returned for each file that was uploaded.
        :rtype: InlineResponse200_65
        """

        Validator(str).validate(payout)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).validate(file)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/files/documents/payout/{{payout}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_header("file", file)
            .add_path("payout", payout)
            .serialize()
            .set_method("POST")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_65._unmap(response)

    @cast_models
    def list_documents_of_payout(
        self,
        payout: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_65:
        """Retrieve details of all supporting document files that were previously uploaded to a specific payout.

        :param payout: ID of the payout. String starting with **payout_**.
        :type payout: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The following parameters are returned for each file that was uploaded.
        :rtype: InlineResponse200_65
        """

        Validator(str).validate(payout)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payouts/{{payout}}/documents/",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("payout", payout)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_65._unmap(response)

    @cast_models
    def delete_documents_of_payout(
        self,
        payout: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_66:
        """Delete all document files that were previously uploaded to a specific payout.

        :param payout: ID of the payout. String starting with **payout_**.
        :type payout: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Returns a **SUCCESS** status.
        :rtype: InlineResponse200_66
        """

        Validator(str).validate(payout)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payouts/{{payout}}/documents/",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("payout", payout)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_66._unmap(response)

    @cast_models
    def get_document_of_payout(
        self,
        payout: str,
        file_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_65:
        """Retrieve details of all supporting document files that were previously uploaded to a specific payout.

        :param payout: ID of the payout. String starting with **payout_**.
        :type payout: str
        :param file_id: ID of the specific document file. UUID.
        :type file_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The following parameters are returned for the file that was uploaded.
        :rtype: InlineResponse200_65
        """

        Validator(str).validate(payout)
        Validator(str).validate(file_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payouts/{{payout}}/documents/{{file_id}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("payout", payout)
            .add_path("file_id", file_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_65._unmap(response)

    @cast_models
    def delete_document_of_payout(
        self,
        payout: str,
        file_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_66:
        """Delete a document file that was previously uploaded to a specific payout.

        :param payout: ID of the payout. String starting with **payout_**.
        :type payout: str
        :param file_id: ID of the specific document file. UUID.
        :type file_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Returns a **SUCCESS** status.
        :rtype: InlineResponse200_66
        """

        Validator(str).validate(payout)
        Validator(str).validate(file_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payouts/{{payout}}/documents/{{file_id}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("payout", payout)
            .add_path("file_id", file_id)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_66._unmap(response)



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .category import Category
from .entity_type import EntityType


@JsonMap({"id_": "id"})
class Beneficiary(BaseModel):
    """Beneficiary

    :param aba: American Bankers Association routing transit number. United States., defaults to None
    :type aba: str, optional
    :param account_number: The person's account number., defaults to None
    :type account_number: str, optional
    :param ach_code: Automated Clearing House (ACH) code., defaults to None
    :type ach_code: str, optional
    :param additional_data: Additional data required by the payout method. Object., defaults to None
    :type additional_data: dict, optional
    :param additional_last_name: The person's second last name., defaults to None
    :type additional_last_name: str, optional
    :param address: Street and number portion of the person's address., defaults to None
    :type address: str, optional
    :param bank_account_holder_name: Name of the account holder., defaults to None
    :type bank_account_holder_name: str, optional
    :param bank_account_type: Type of bank account, such as checking or savings., defaults to None
    :type bank_account_type: str, optional
    :param bank_address: Street and number portion of the bank's address., defaults to None
    :type bank_address: str, optional
    :param bank_branch_code: Identifier for the bank branch., defaults to None
    :type bank_branch_code: str, optional
    :param bank_branch_name: Name of the bank branch., defaults to None
    :type bank_branch_name: str, optional
    :param bank_city: City portion of the bank's address., defaults to None
    :type bank_city: str, optional
    :param bank_code: Domestic identifier of the bank., defaults to None
    :type bank_code: str, optional
    :param bank_country: Two-letter ISO 3166-1 ALPHA-2 code for the bank's country., defaults to None
    :type bank_country: str, optional
    :param bank_name: Name of the bank., defaults to None
    :type bank_name: str, optional
    :param bank_province: State or province portion of the bank's address., defaults to None
    :type bank_province: str, optional
    :param beneficiary_bic_local: NA., defaults to None
    :type beneficiary_bic_local: str, optional
    :param beneficiary_relationship: Relationship between the sender and the beneficiary., defaults to None
    :type beneficiary_relationship: str, optional
    :param bic_swift: BIC (SWIFT) code for the bank account., defaults to None
    :type bic_swift: str, optional
    :param bsb_code: Bank State Branch code for Australia., defaults to None
    :type bsb_code: str, optional
    :param card_expiration_month: The cardâ€™s expiration month., defaults to None
    :type card_expiration_month: str, optional
    :param card_expiration_year: The cardâ€™s expiration year., defaults to None
    :type card_expiration_year: str, optional
    :param card_number: The card number., defaults to None
    :type card_number: str, optional
    :param category: category, defaults to None
    :type category: Category, optional
    :param city: City portion of the address., defaults to None
    :type city: str, optional
    :param clabe: Standardized bank code for Mexico., defaults to None
    :type clabe: str, optional
    :param cnaps: China National Advanced Payments System (CNAPS) code number., defaults to None
    :type cnaps: str, optional
    :param company_name: Name of the beneficiary company. Relevant when entity_type is company., defaults to None
    :type company_name: str, optional
    :param confirmation_required: Indicates whether the beneficiary must respond to accept or decline a payout. Default is false. Relevant to the rapyd_ewallet payout method type, defaults to None
    :type confirmation_required: bool, optional
    :param country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code., defaults to None
    :type country: str, optional
    :param currency: currency, defaults to None
    :type currency: str, optional
    :param date_of_birth: The person's date of birth., defaults to None
    :type date_of_birth: str, optional
    :param date_of_incorporation: Date of incorporation of the corporation., defaults to None
    :type date_of_incorporation: str, optional
    :param default_payout_method_type: The type of payout method for the beneficiary. The two-letter prefix must match the beneficiary country code., defaults to None
    :type default_payout_method_type: str, optional
    :param email: The person's email address., defaults to None
    :type email: str, optional
    :param entity_type: entity_type, defaults to None
    :type entity_type: EntityType, optional
    :param ewallet: ID of the wallet that the money is transferred to. String starting with **ewallet_**. Relevant to payout to Rapyd Wallet., defaults to None
    :type ewallet: str, optional
    :param first_name: First name of the beneficiary. Relevant when **entity_type** is individual., defaults to None
    :type first_name: str, optional
    :param iban: International Bank Account Number., defaults to None
    :type iban: str, optional
    :param id_: ID of the 'beneficiary' object. String starting with **beneficiary_**. Response only., defaults to None
    :type id_: str, optional
    :param id_date_of_issue: The date of issuance of the person's identification document., defaults to None
    :type id_date_of_issue: str, optional
    :param id_issue_authority: The government agency that issued the person's identification document., defaults to None
    :type id_issue_authority: str, optional
    :param id_issue_location: The location where the person's identification document was issued., defaults to None
    :type id_issue_location: str, optional
    :param identification_type: Type of identification document for the beneficiary. When entity_type is company, this field must be company_registered_number. When entity_type is individual, one of the following values: * drivers_license * identification_id * international_passport * residence_permit* social_security * work_permit , defaults to None
    :type identification_type: str, optional
    :param identification_value: identification number on the document mentioned in identification_type., defaults to None
    :type identification_value: str, optional
    :param ifsc: Indian Financial System Code for a bank branch., defaults to None
    :type ifsc: str, optional
    :param issuer_code: NA., defaults to None
    :type issuer_code: str, optional
    :param last_name: Last name of the beneficiary. Relevant when `entity_type` is individual., defaults to None
    :type last_name: str, optional
    :param merchant_reference_id: ID defined by the merchant. Rapyd does not validate this value to enforce uniqueness., defaults to None
    :type merchant_reference_id: str, optional
    :param middle_name: The person's middle name., defaults to None
    :type middle_name: str, optional
    :param mobile_number: Mobile phone number in E.164 format., defaults to None
    :type mobile_number: str, optional
    :param name: The person's full name., defaults to None
    :type name: str, optional
    :param nationality: The person's nationality., defaults to None
    :type nationality: str, optional
    :param payment_type: Additional classification of the type of payment, as required by the bank. Possible values: priority , regular, defaults to None
    :type payment_type: str, optional
    :param phone_country_code: The country code of the person's phone., defaults to None
    :type phone_country_code: str, optional
    :param phone_number: Phone number in E.164 format., defaults to None
    :type phone_number: str, optional
    :param postcode: Postal code portion of the address., defaults to None
    :type postcode: str, optional
    :param province: Province portion of the address., defaults to None
    :type province: str, optional
    :param purpose_code: Purpose of the transaction., defaults to None
    :type purpose_code: str, optional
    :param routing_code_type_1: NA., defaults to None
    :type routing_code_type_1: str, optional
    :param routing_code_type_2: NA., defaults to None
    :type routing_code_type_2: str, optional
    :param routing_code_type_3: NA., defaults to None
    :type routing_code_type_3: str, optional
    :param routing_code_value_1: NA., defaults to None
    :type routing_code_value_1: str, optional
    :param routing_code_value_2: NA., defaults to None
    :type routing_code_value_2: str, optional
    :param routing_code_value_3: NA., defaults to None
    :type routing_code_value_3: str, optional
    :param sort_code: Routing number for the UK., defaults to None
    :type sort_code: str, optional
    :param state: State portion of the address., defaults to None
    :type state: str, optional
    :param statement_narrative: Additional description suitable for a customer transaction statement., defaults to None
    :type statement_narrative: str, optional
    :param suburb: District of the sender's or beneficiary's city., defaults to None
    :type suburb: str, optional
    :param tax_id: Tax identification number., defaults to None
    :type tax_id: str, optional
    :param vpa: Virtual Payment Address., defaults to None
    :type vpa: str, optional
    """

    def __init__(
        self,
        aba: str = None,
        account_number: str = None,
        ach_code: str = None,
        additional_data: dict = None,
        additional_last_name: str = None,
        address: str = None,
        bank_account_holder_name: str = None,
        bank_account_type: str = None,
        bank_address: str = None,
        bank_branch_code: str = None,
        bank_branch_name: str = None,
        bank_city: str = None,
        bank_code: str = None,
        bank_country: str = None,
        bank_name: str = None,
        bank_province: str = None,
        beneficiary_bic_local: str = None,
        beneficiary_relationship: str = None,
        bic_swift: str = None,
        bsb_code: str = None,
        card_expiration_month: str = None,
        card_expiration_year: str = None,
        card_number: str = None,
        category: Category = None,
        city: str = None,
        clabe: str = None,
        cnaps: str = None,
        company_name: str = None,
        confirmation_required: bool = None,
        country: str = None,
        currency: str = None,
        date_of_birth: str = None,
        date_of_incorporation: str = None,
        default_payout_method_type: str = None,
        email: str = None,
        entity_type: EntityType = None,
        ewallet: str = None,
        first_name: str = None,
        iban: str = None,
        id_: str = None,
        id_date_of_issue: str = None,
        id_issue_authority: str = None,
        id_issue_location: str = None,
        identification_type: str = None,
        identification_value: str = None,
        ifsc: str = None,
        issuer_code: str = None,
        last_name: str = None,
        merchant_reference_id: str = None,
        middle_name: str = None,
        mobile_number: str = None,
        name: str = None,
        nationality: str = None,
        payment_type: str = None,
        phone_country_code: str = None,
        phone_number: str = None,
        postcode: str = None,
        province: str = None,
        purpose_code: str = None,
        routing_code_type_1: str = None,
        routing_code_type_2: str = None,
        routing_code_type_3: str = None,
        routing_code_value_1: str = None,
        routing_code_value_2: str = None,
        routing_code_value_3: str = None,
        sort_code: str = None,
        state: str = None,
        statement_narrative: str = None,
        suburb: str = None,
        tax_id: str = None,
        vpa: str = None,
    ):
        """Beneficiary

        :param aba: American Bankers Association routing transit number. United States., defaults to None
        :type aba: str, optional
        :param account_number: The person's account number., defaults to None
        :type account_number: str, optional
        :param ach_code: Automated Clearing House (ACH) code., defaults to None
        :type ach_code: str, optional
        :param additional_data: Additional data required by the payout method. Object., defaults to None
        :type additional_data: dict, optional
        :param additional_last_name: The person's second last name., defaults to None
        :type additional_last_name: str, optional
        :param address: Street and number portion of the person's address., defaults to None
        :type address: str, optional
        :param bank_account_holder_name: Name of the account holder., defaults to None
        :type bank_account_holder_name: str, optional
        :param bank_account_type: Type of bank account, such as checking or savings., defaults to None
        :type bank_account_type: str, optional
        :param bank_address: Street and number portion of the bank's address., defaults to None
        :type bank_address: str, optional
        :param bank_branch_code: Identifier for the bank branch., defaults to None
        :type bank_branch_code: str, optional
        :param bank_branch_name: Name of the bank branch., defaults to None
        :type bank_branch_name: str, optional
        :param bank_city: City portion of the bank's address., defaults to None
        :type bank_city: str, optional
        :param bank_code: Domestic identifier of the bank., defaults to None
        :type bank_code: str, optional
        :param bank_country: Two-letter ISO 3166-1 ALPHA-2 code for the bank's country., defaults to None
        :type bank_country: str, optional
        :param bank_name: Name of the bank., defaults to None
        :type bank_name: str, optional
        :param bank_province: State or province portion of the bank's address., defaults to None
        :type bank_province: str, optional
        :param beneficiary_bic_local: NA., defaults to None
        :type beneficiary_bic_local: str, optional
        :param beneficiary_relationship: Relationship between the sender and the beneficiary., defaults to None
        :type beneficiary_relationship: str, optional
        :param bic_swift: BIC (SWIFT) code for the bank account., defaults to None
        :type bic_swift: str, optional
        :param bsb_code: Bank State Branch code for Australia., defaults to None
        :type bsb_code: str, optional
        :param card_expiration_month: The cardâ€™s expiration month., defaults to None
        :type card_expiration_month: str, optional
        :param card_expiration_year: The cardâ€™s expiration year., defaults to None
        :type card_expiration_year: str, optional
        :param card_number: The card number., defaults to None
        :type card_number: str, optional
        :param category: category, defaults to None
        :type category: Category, optional
        :param city: City portion of the address., defaults to None
        :type city: str, optional
        :param clabe: Standardized bank code for Mexico., defaults to None
        :type clabe: str, optional
        :param cnaps: China National Advanced Payments System (CNAPS) code number., defaults to None
        :type cnaps: str, optional
        :param company_name: Name of the beneficiary company. Relevant when entity_type is company., defaults to None
        :type company_name: str, optional
        :param confirmation_required: Indicates whether the beneficiary must respond to accept or decline a payout. Default is false. Relevant to the rapyd_ewallet payout method type, defaults to None
        :type confirmation_required: bool, optional
        :param country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code., defaults to None
        :type country: str, optional
        :param currency: currency, defaults to None
        :type currency: str, optional
        :param date_of_birth: The person's date of birth., defaults to None
        :type date_of_birth: str, optional
        :param date_of_incorporation: Date of incorporation of the corporation., defaults to None
        :type date_of_incorporation: str, optional
        :param default_payout_method_type: The type of payout method for the beneficiary. The two-letter prefix must match the beneficiary country code., defaults to None
        :type default_payout_method_type: str, optional
        :param email: The person's email address., defaults to None
        :type email: str, optional
        :param entity_type: entity_type, defaults to None
        :type entity_type: EntityType, optional
        :param ewallet: ID of the wallet that the money is transferred to. String starting with **ewallet_**. Relevant to payout to Rapyd Wallet., defaults to None
        :type ewallet: str, optional
        :param first_name: First name of the beneficiary. Relevant when **entity_type** is individual., defaults to None
        :type first_name: str, optional
        :param iban: International Bank Account Number., defaults to None
        :type iban: str, optional
        :param id_: ID of the 'beneficiary' object. String starting with **beneficiary_**. Response only., defaults to None
        :type id_: str, optional
        :param id_date_of_issue: The date of issuance of the person's identification document., defaults to None
        :type id_date_of_issue: str, optional
        :param id_issue_authority: The government agency that issued the person's identification document., defaults to None
        :type id_issue_authority: str, optional
        :param id_issue_location: The location where the person's identification document was issued., defaults to None
        :type id_issue_location: str, optional
        :param identification_type: Type of identification document for the beneficiary. When entity_type is company, this field must be company_registered_number. When entity_type is individual, one of the following values: * drivers_license * identification_id * international_passport * residence_permit* social_security * work_permit , defaults to None
        :type identification_type: str, optional
        :param identification_value: identification number on the document mentioned in identification_type., defaults to None
        :type identification_value: str, optional
        :param ifsc: Indian Financial System Code for a bank branch., defaults to None
        :type ifsc: str, optional
        :param issuer_code: NA., defaults to None
        :type issuer_code: str, optional
        :param last_name: Last name of the beneficiary. Relevant when `entity_type` is individual., defaults to None
        :type last_name: str, optional
        :param merchant_reference_id: ID defined by the merchant. Rapyd does not validate this value to enforce uniqueness., defaults to None
        :type merchant_reference_id: str, optional
        :param middle_name: The person's middle name., defaults to None
        :type middle_name: str, optional
        :param mobile_number: Mobile phone number in E.164 format., defaults to None
        :type mobile_number: str, optional
        :param name: The person's full name., defaults to None
        :type name: str, optional
        :param nationality: The person's nationality., defaults to None
        :type nationality: str, optional
        :param payment_type: Additional classification of the type of payment, as required by the bank. Possible values: priority , regular, defaults to None
        :type payment_type: str, optional
        :param phone_country_code: The country code of the person's phone., defaults to None
        :type phone_country_code: str, optional
        :param phone_number: Phone number in E.164 format., defaults to None
        :type phone_number: str, optional
        :param postcode: Postal code portion of the address., defaults to None
        :type postcode: str, optional
        :param province: Province portion of the address., defaults to None
        :type province: str, optional
        :param purpose_code: Purpose of the transaction., defaults to None
        :type purpose_code: str, optional
        :param routing_code_type_1: NA., defaults to None
        :type routing_code_type_1: str, optional
        :param routing_code_type_2: NA., defaults to None
        :type routing_code_type_2: str, optional
        :param routing_code_type_3: NA., defaults to None
        :type routing_code_type_3: str, optional
        :param routing_code_value_1: NA., defaults to None
        :type routing_code_value_1: str, optional
        :param routing_code_value_2: NA., defaults to None
        :type routing_code_value_2: str, optional
        :param routing_code_value_3: NA., defaults to None
        :type routing_code_value_3: str, optional
        :param sort_code: Routing number for the UK., defaults to None
        :type sort_code: str, optional
        :param state: State portion of the address., defaults to None
        :type state: str, optional
        :param statement_narrative: Additional description suitable for a customer transaction statement., defaults to None
        :type statement_narrative: str, optional
        :param suburb: District of the sender's or beneficiary's city., defaults to None
        :type suburb: str, optional
        :param tax_id: Tax identification number., defaults to None
        :type tax_id: str, optional
        :param vpa: Virtual Payment Address., defaults to None
        :type vpa: str, optional
        """
        self.aba = self._define_str("aba", aba, nullable=True)
        self.account_number = self._define_str(
            "account_number", account_number, nullable=True
        )
        self.ach_code = self._define_str("ach_code", ach_code, nullable=True)
        self.additional_data = additional_data
        self.additional_last_name = self._define_str(
            "additional_last_name", additional_last_name, nullable=True
        )
        self.address = self._define_str("address", address, nullable=True)
        self.bank_account_holder_name = self._define_str(
            "bank_account_holder_name", bank_account_holder_name, nullable=True
        )
        self.bank_account_type = self._define_str(
            "bank_account_type", bank_account_type, nullable=True
        )
        self.bank_address = self._define_str(
            "bank_address", bank_address, nullable=True
        )
        self.bank_branch_code = self._define_str(
            "bank_branch_code", bank_branch_code, nullable=True
        )
        self.bank_branch_name = self._define_str(
            "bank_branch_name", bank_branch_name, nullable=True
        )
        self.bank_city = self._define_str("bank_city", bank_city, nullable=True)
        self.bank_code = self._define_str("bank_code", bank_code, nullable=True)
        self.bank_country = self._define_str(
            "bank_country", bank_country, nullable=True
        )
        self.bank_name = self._define_str("bank_name", bank_name, nullable=True)
        self.bank_province = self._define_str(
            "bank_province", bank_province, nullable=True
        )
        self.beneficiary_bic_local = self._define_str(
            "beneficiary_bic_local", beneficiary_bic_local, nullable=True
        )
        self.beneficiary_relationship = self._define_str(
            "beneficiary_relationship", beneficiary_relationship, nullable=True
        )
        self.bic_swift = self._define_str("bic_swift", bic_swift, nullable=True)
        self.bsb_code = self._define_str("bsb_code", bsb_code, nullable=True)
        self.card_expiration_month = self._define_str(
            "card_expiration_month", card_expiration_month, nullable=True
        )
        self.card_expiration_year = self._define_str(
            "card_expiration_year", card_expiration_year, nullable=True
        )
        self.card_number = self._define_str("card_number", card_number, nullable=True)
        self.category = (
            self._enum_matching(category, Category.list(), "category")
            if category
            else None
        )
        self.city = self._define_str("city", city, nullable=True)
        self.clabe = self._define_str("clabe", clabe, nullable=True)
        self.cnaps = self._define_str("cnaps", cnaps, nullable=True)
        self.company_name = self._define_str(
            "company_name", company_name, nullable=True
        )
        self.confirmation_required = confirmation_required
        self.country = self._define_str("country", country, nullable=True)
        self.currency = self._define_str(
            "currency",
            currency,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.date_of_birth = self._define_str(
            "date_of_birth", date_of_birth, nullable=True
        )
        self.date_of_incorporation = self._define_str(
            "date_of_incorporation", date_of_incorporation, nullable=True
        )
        self.default_payout_method_type = self._define_str(
            "default_payout_method_type", default_payout_method_type, nullable=True
        )
        self.email = self._define_str("email", email, nullable=True)
        self.entity_type = (
            self._enum_matching(entity_type, EntityType.list(), "entity_type")
            if entity_type
            else None
        )
        self.ewallet = self._define_str("ewallet", ewallet, nullable=True)
        self.first_name = self._define_str("first_name", first_name, nullable=True)
        self.iban = self._define_str("iban", iban, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.id_date_of_issue = self._define_str(
            "id_date_of_issue", id_date_of_issue, nullable=True
        )
        self.id_issue_authority = self._define_str(
            "id_issue_authority", id_issue_authority, nullable=True
        )
        self.id_issue_location = self._define_str(
            "id_issue_location", id_issue_location, nullable=True
        )
        self.identification_type = self._define_str(
            "identification_type", identification_type, nullable=True
        )
        self.identification_value = self._define_str(
            "identification_value", identification_value, nullable=True
        )
        self.ifsc = self._define_str("ifsc", ifsc, nullable=True)
        self.issuer_code = self._define_str("issuer_code", issuer_code, nullable=True)
        self.last_name = self._define_str("last_name", last_name, nullable=True)
        self.merchant_reference_id = self._define_str(
            "merchant_reference_id", merchant_reference_id, nullable=True
        )
        self.middle_name = self._define_str("middle_name", middle_name, nullable=True)
        self.mobile_number = self._define_str(
            "mobile_number", mobile_number, nullable=True
        )
        self.name = self._define_str("name", name, nullable=True)
        self.nationality = self._define_str("nationality", nationality, nullable=True)
        self.payment_type = self._define_str(
            "payment_type", payment_type, nullable=True
        )
        self.phone_country_code = self._define_str(
            "phone_country_code", phone_country_code, nullable=True
        )
        self.phone_number = self._define_str(
            "phone_number", phone_number, nullable=True
        )
        self.postcode = self._define_str("postcode", postcode, nullable=True)
        self.province = self._define_str("province", province, nullable=True)
        self.purpose_code = self._define_str(
            "purpose_code", purpose_code, nullable=True
        )
        self.routing_code_type_1 = self._define_str(
            "routing_code_type_1", routing_code_type_1, nullable=True
        )
        self.routing_code_type_2 = self._define_str(
            "routing_code_type_2", routing_code_type_2, nullable=True
        )
        self.routing_code_type_3 = self._define_str(
            "routing_code_type_3", routing_code_type_3, nullable=True
        )
        self.routing_code_value_1 = self._define_str(
            "routing_code_value_1", routing_code_value_1, nullable=True
        )
        self.routing_code_value_2 = self._define_str(
            "routing_code_value_2", routing_code_value_2, nullable=True
        )
        self.routing_code_value_3 = self._define_str(
            "routing_code_value_3", routing_code_value_3, nullable=True
        )
        self.sort_code = self._define_str("sort_code", sort_code, nullable=True)
        self.state = self._define_str("state", state, nullable=True)
        self.statement_narrative = self._define_str(
            "statement_narrative", statement_narrative, nullable=True
        )
        self.suburb = self._define_str("suburb", suburb, nullable=True)
        self.tax_id = self._define_str("tax_id", tax_id, nullable=True)
        self.vpa = self._define_str("vpa", vpa, nullable=True)



from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .inline_response_200_117_data_merchant_customer_support import (
    InlineResponse200_117DataMerchantCustomerSupport,
)


class InlineResponse200_117DataRequestType(Enum):
    """An enumeration representing different categories.

    :cvar STORE: "store"
    :vartype STORE: str
    :cvar VERIFY: "verify"
    :vartype VERIFY: str
    """

    STORE = "store"
    VERIFY = "verify"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                InlineResponse200_117DataRequestType._member_map_.values(),
            )
        )


class InlineResponse200_117DataStatus(Enum):
    """An enumeration representing different categories.

    :cvar NEW: "NEW"
    :vartype NEW: str
    :cvar DON: "DON"
    :vartype DON: str
    :cvar EXP: "EXP"
    :vartype EXP: str
    """

    NEW = "NEW"
    DON = "DON"
    EXP = "EXP"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value, InlineResponse200_117DataStatus._member_map_.values()
            )
        )


@JsonMap({"id_": "id"})
class InlineResponse200_117Data(BaseModel):
    """Retrieve the Rapyd ID and merchant reference ID.

    :param cancel_url: URL where the customer is redirected after pressing **Back to Website** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
    :type cancel_url: str, optional
    :param complete_url: URL where the customer is redirected after pressing **Close** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
    :type complete_url: str, optional
    :param contact: ID of the wallet contact. String starting with **cont_**., defaults to None
    :type contact: str, optional
    :param country: The two-letter ISO 3166-1 ALPHA-2 code for the country of the identification document., defaults to None
    :type country: str, optional
    :param id_: ID of the Hosted Page Identity Verification object, a string starting with **hp_idv_**., defaults to None
    :type id_: str, optional
    :param language: Determines the default language of the hosted page. For a list of values, see [List Supported Languages](https://docs.rapyd.net/en/list-supported-languages.html). \<BR\> * When this parameter is null, the language of the user's browser is used. \<BR\> * If the language of the user's browser cannot be determined, the default language is English., defaults to None
    :type language: str, optional
    :param merchant_alias: Client's name., defaults to None
    :type merchant_alias: str, optional
    :param merchant_color: Color of the call-to-action (CTA) button on the hosted page.\<BR\> To configure this field, use the Client Portal., defaults to None
    :type merchant_color: str, optional
    :param merchant_customer_support: Contains details of the client’s customer support. To configure these fields, use the Client Portal., defaults to None
    :type merchant_customer_support: InlineResponse200_117DataMerchantCustomerSupport, optional
    :param merchant_logo: URL for the image of the client's logo.\<BR\> To configure this field, use the Client Portal., defaults to None
    :type merchant_logo: str, optional
    :param merchant_privacy_policy: URL for the terms and conditions of the agreement between the client and the client’s customers.\<BR\>To configure this field, use the Client Portal., defaults to None
    :type merchant_privacy_policy: str, optional
    :param merchant_terms: URL for the client's terms and conditions. To configure this field, use the Client Porta, defaults to None
    :type merchant_terms: str, optional
    :param merchant_website: The URL where the customer is redirected after exiting the hosted page. Relevant when `cancel_url`, `complete_url` or both fields are not set. To configure this field, use the Client Portal., defaults to None
    :type merchant_website: str, optional
    :param page_expiration: End of the time when the customer can use the hosted page, in Unix time. If `page_expiration` is not set, the hosted page expires 14 days after creation.\<BR\>**Range**: 1 minute to 30 days., defaults to None
    :type page_expiration: float, optional
    :param redirect_url: URL of the hosted page that is shown to the customer., defaults to None
    :type redirect_url: str, optional
    :param reference_id: ID of the identity verification request. Must be unique for each request. Defined by the client., defaults to None
    :type reference_id: str, optional
    :param request_type: Determines the action that is taken on the request. One of the following:\<BR\>* **store** - Store the images on the Rapyd platform.\<BR\>* **verify** - Verify the identity of the person in the images., defaults to None
    :type request_type: InlineResponse200_117DataRequestType, optional
    :param status: Status of the hosted page. One of the following:\<BR\>* **NEW** - The hosted page was created.\<BR\>* **DON** - Done. The identity verification request was submitted.\<BR\>* **EXP** - Expired. The hosted page expired., defaults to None
    :type status: InlineResponse200_117DataStatus, optional
    """

    def __init__(
        self,
        cancel_url: str = None,
        complete_url: str = None,
        contact: str = None,
        country: str = None,
        id_: str = None,
        language: str = None,
        merchant_alias: str = None,
        merchant_color: str = None,
        merchant_customer_support: InlineResponse200_117DataMerchantCustomerSupport = None,
        merchant_logo: str = None,
        merchant_privacy_policy: str = None,
        merchant_terms: str = None,
        merchant_website: str = None,
        page_expiration: float = None,
        redirect_url: str = None,
        reference_id: str = None,
        request_type: InlineResponse200_117DataRequestType = None,
        status: InlineResponse200_117DataStatus = None,
    ):
        """Retrieve the Rapyd ID and merchant reference ID.

        :param cancel_url: URL where the customer is redirected after pressing **Back to Website** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
        :type cancel_url: str, optional
        :param complete_url: URL where the customer is redirected after pressing **Close** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
        :type complete_url: str, optional
        :param contact: ID of the wallet contact. String starting with **cont_**., defaults to None
        :type contact: str, optional
        :param country: The two-letter ISO 3166-1 ALPHA-2 code for the country of the identification document., defaults to None
        :type country: str, optional
        :param id_: ID of the Hosted Page Identity Verification object, a string starting with **hp_idv_**., defaults to None
        :type id_: str, optional
        :param language: Determines the default language of the hosted page. For a list of values, see [List Supported Languages](https://docs.rapyd.net/en/list-supported-languages.html). \<BR\> * When this parameter is null, the language of the user's browser is used. \<BR\> * If the language of the user's browser cannot be determined, the default language is English., defaults to None
        :type language: str, optional
        :param merchant_alias: Client's name., defaults to None
        :type merchant_alias: str, optional
        :param merchant_color: Color of the call-to-action (CTA) button on the hosted page.\<BR\> To configure this field, use the Client Portal., defaults to None
        :type merchant_color: str, optional
        :param merchant_customer_support: Contains details of the client’s customer support. To configure these fields, use the Client Portal., defaults to None
        :type merchant_customer_support: InlineResponse200_117DataMerchantCustomerSupport, optional
        :param merchant_logo: URL for the image of the client's logo.\<BR\> To configure this field, use the Client Portal., defaults to None
        :type merchant_logo: str, optional
        :param merchant_privacy_policy: URL for the terms and conditions of the agreement between the client and the client’s customers.\<BR\>To configure this field, use the Client Portal., defaults to None
        :type merchant_privacy_policy: str, optional
        :param merchant_terms: URL for the client's terms and conditions. To configure this field, use the Client Porta, defaults to None
        :type merchant_terms: str, optional
        :param merchant_website: The URL where the customer is redirected after exiting the hosted page. Relevant when `cancel_url`, `complete_url` or both fields are not set. To configure this field, use the Client Portal., defaults to None
        :type merchant_website: str, optional
        :param page_expiration: End of the time when the customer can use the hosted page, in Unix time. If `page_expiration` is not set, the hosted page expires 14 days after creation.\<BR\>**Range**: 1 minute to 30 days., defaults to None
        :type page_expiration: float, optional
        :param redirect_url: URL of the hosted page that is shown to the customer., defaults to None
        :type redirect_url: str, optional
        :param reference_id: ID of the identity verification request. Must be unique for each request. Defined by the client., defaults to None
        :type reference_id: str, optional
        :param request_type: Determines the action that is taken on the request. One of the following:\<BR\>* **store** - Store the images on the Rapyd platform.\<BR\>* **verify** - Verify the identity of the person in the images., defaults to None
        :type request_type: InlineResponse200_117DataRequestType, optional
        :param status: Status of the hosted page. One of the following:\<BR\>* **NEW** - The hosted page was created.\<BR\>* **DON** - Done. The identity verification request was submitted.\<BR\>* **EXP** - Expired. The hosted page expired., defaults to None
        :type status: InlineResponse200_117DataStatus, optional
        """
        self.cancel_url = self._define_str("cancel_url", cancel_url, nullable=True)
        self.complete_url = self._define_str(
            "complete_url", complete_url, nullable=True
        )
        self.contact = self._define_str("contact", contact, nullable=True)
        self.country = self._define_str("country", country, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.language = self._define_str("language", language, nullable=True)
        self.merchant_alias = self._define_str(
            "merchant_alias", merchant_alias, nullable=True
        )
        self.merchant_color = self._define_str(
            "merchant_color", merchant_color, nullable=True
        )
        self.merchant_customer_support = self._define_object(
            merchant_customer_support, InlineResponse200_117DataMerchantCustomerSupport
        )
        self.merchant_logo = self._define_str(
            "merchant_logo", merchant_logo, nullable=True
        )
        self.merchant_privacy_policy = self._define_str(
            "merchant_privacy_policy", merchant_privacy_policy, nullable=True
        )
        self.merchant_terms = self._define_str(
            "merchant_terms", merchant_terms, nullable=True
        )
        self.merchant_website = self._define_str(
            "merchant_website", merchant_website, nullable=True
        )
        self.page_expiration = self._define_number(
            "page_expiration", page_expiration, nullable=True
        )
        self.redirect_url = self._define_str(
            "redirect_url", redirect_url, nullable=True
        )
        self.reference_id = self._define_str(
            "reference_id", reference_id, nullable=True
        )
        self.request_type = (
            self._enum_matching(
                request_type,
                InlineResponse200_117DataRequestType.list(),
                "request_type",
            )
            if request_type
            else None
        )
        self.status = (
            self._enum_matching(
                status, InlineResponse200_117DataStatus.list(), "status"
            )
            if status
            else None
        )



from __future__ import annotations
from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .payment_amount_range_per_currency_inner import PaymentAmountRangePerCurrencyInner
from .category import Category
from .field_1 import Field1


class PaymentMethodTypePaymentFlowType(Enum):
    """An enumeration representing different categories.

    :cvar DIRECT: "direct"
    :vartype DIRECT: str
    :cvar EWALLET: "eWallet"
    :vartype EWALLET: str
    :cvar EWALLETPAYOUT: "ewallet_payout"
    :vartype EWALLETPAYOUT: str
    :cvar CARD: "card"
    :vartype CARD: str
    :cvar REDIRECTURL: "redirect_url"
    :vartype REDIRECTURL: str
    :cvar CASH: "cash"
    :vartype CASH: str
    :cvar BANKTRANSFER: "bank transfer"
    :vartype BANKTRANSFER: str
    """

    DIRECT = "direct"
    EWALLET = "eWallet"
    EWALLETPAYOUT = "ewallet_payout"
    CARD = "card"
    REDIRECTURL = "redirect_url"
    CASH = "cash"
    BANKTRANSFER = "bank transfer"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                PaymentMethodTypePaymentFlowType._member_map_.values(),
            )
        )


@JsonMap({"type_": "type"})
class PaymentMethodType(BaseModel):
    """A type of payment method that a customer can use for making payments.

    :param amount_range_per_currency: Indicates the amount range for the payment method's currencies. Each object contains the following fields:* currency - Three-letter ISO 4217 format of currency* maximum_amount - The maximum payment amount* minimum_amount - The minimum payment amount, defaults to None
    :type amount_range_per_currency: List[PaymentAmountRangePerCurrencyInner], optional
    :param category: category, defaults to None
    :type category: Category, optional
    :param country: country, defaults to None
    :type country: str, optional
    :param currencies: currencies, defaults to None
    :type currencies: List[str], optional
    :param fields: Payment Method Type required field, defaults to None
    :type fields: List[Field1], optional
    :param image: A URL to the image of the icon for the type of payment method. Response only, defaults to None
    :type image: str, optional
    :param is_cancelable: Indicates whether a payment made with this payment method can be canceled. Response only, defaults to None
    :type is_cancelable: bool, optional
    :param is_expirable: Indicates whether the merchant can set an expiration time for the customer to complete the payment. Response only, defaults to None
    :type is_expirable: bool, optional
    :param is_online: Indicates whether the payment is completed immediately online. Response only, defaults to None
    :type is_online: bool, optional
    :param is_refundable: Indicates whether the payment method type supports refunds, defaults to None
    :type is_refundable: bool, optional
    :param is_tokenizable: Indicates whether the token of the payment method can be used in a collect operation, defaults to None
    :type is_tokenizable: bool, optional
    :param is_virtual: Indicates whether a Web-based version of the payment method type exists, defaults to None
    :type is_virtual: bool, optional
    :param maximum_expiration_seconds: The maximum time (in seconds) that the merchant can set for completing the payment. Relevant when is_expirable is true, defaults to None
    :type maximum_expiration_seconds: float, optional
    :param minimum_expiration_seconds: The minimum time (in seconds) that the merchant can set for completing the payment. Relevant when is_expirable is true, defaults to None
    :type minimum_expiration_seconds: float, optional
    :param multiple_overage_allowed: Indicates whether multiple overage charges are allowed for this payment method type, defaults to None
    :type multiple_overage_allowed: bool, optional
    :param name: The name of the payment method, in user-friendly terms. For example, Ireland Visa card. Response only, defaults to None
    :type name: str, optional
    :param payment_flow_type: payment_flow_type, defaults to None
    :type payment_flow_type: PaymentMethodTypePaymentFlowType, optional
    :param payment_options: payment_options, defaults to None
    :type payment_options: List[Field1], optional
    :param status: Indicates the status of the payment method. One of the following value is 1 means the payment_method_type is Valid, defaults to None
    :type status: str, optional
    :param supported_digital_wallet_providers: Describes the digital wallet providers that support the payment method. These providers may include **apple_pay** and **google_pay**., defaults to None
    :type supported_digital_wallet_providers: List[str], optional
    :param type_: Type of the payment method. For example, it_visa_card, defaults to None
    :type type_: str, optional
    :param virtual_payment_method_type: Indicates the name of the Web-based version of this payment method type, defaults to None
    :type virtual_payment_method_type: str, optional
    :param is_restricted: Indicates if the payment method type restricted or not., defaults to None
    :type is_restricted: bool, optional
    :param supports_subscription: Indicates if the payment method type supports subscription., defaults to None
    :type supports_subscription: bool, optional
    """

    def __init__(
        self,
        amount_range_per_currency: List[PaymentAmountRangePerCurrencyInner] = None,
        category: Category = None,
        country: str = None,
        currencies: List[str] = None,
        fields: List[Field1] = None,
        image: str = None,
        is_cancelable: bool = None,
        is_expirable: bool = None,
        is_online: bool = None,
        is_refundable: bool = None,
        is_tokenizable: bool = None,
        is_virtual: bool = None,
        maximum_expiration_seconds: float = None,
        minimum_expiration_seconds: float = None,
        multiple_overage_allowed: bool = None,
        name: str = None,
        payment_flow_type: PaymentMethodTypePaymentFlowType = None,
        payment_options: List[Field1] = None,
        status: str = None,
        supported_digital_wallet_providers: List[str] = None,
        type_: str = None,
        virtual_payment_method_type: str = None,
        is_restricted: bool = None,
        supports_subscription: bool = None,
    ):
        """A type of payment method that a customer can use for making payments.

        :param amount_range_per_currency: Indicates the amount range for the payment method's currencies. Each object contains the following fields:* currency - Three-letter ISO 4217 format of currency* maximum_amount - The maximum payment amount* minimum_amount - The minimum payment amount, defaults to None
        :type amount_range_per_currency: List[PaymentAmountRangePerCurrencyInner], optional
        :param category: category, defaults to None
        :type category: Category, optional
        :param country: country, defaults to None
        :type country: str, optional
        :param currencies: currencies, defaults to None
        :type currencies: List[str], optional
        :param fields: Payment Method Type required field, defaults to None
        :type fields: List[Field1], optional
        :param image: A URL to the image of the icon for the type of payment method. Response only, defaults to None
        :type image: str, optional
        :param is_cancelable: Indicates whether a payment made with this payment method can be canceled. Response only, defaults to None
        :type is_cancelable: bool, optional
        :param is_expirable: Indicates whether the merchant can set an expiration time for the customer to complete the payment. Response only, defaults to None
        :type is_expirable: bool, optional
        :param is_online: Indicates whether the payment is completed immediately online. Response only, defaults to None
        :type is_online: bool, optional
        :param is_refundable: Indicates whether the payment method type supports refunds, defaults to None
        :type is_refundable: bool, optional
        :param is_tokenizable: Indicates whether the token of the payment method can be used in a collect operation, defaults to None
        :type is_tokenizable: bool, optional
        :param is_virtual: Indicates whether a Web-based version of the payment method type exists, defaults to None
        :type is_virtual: bool, optional
        :param maximum_expiration_seconds: The maximum time (in seconds) that the merchant can set for completing the payment. Relevant when is_expirable is true, defaults to None
        :type maximum_expiration_seconds: float, optional
        :param minimum_expiration_seconds: The minimum time (in seconds) that the merchant can set for completing the payment. Relevant when is_expirable is true, defaults to None
        :type minimum_expiration_seconds: float, optional
        :param multiple_overage_allowed: Indicates whether multiple overage charges are allowed for this payment method type, defaults to None
        :type multiple_overage_allowed: bool, optional
        :param name: The name of the payment method, in user-friendly terms. For example, Ireland Visa card. Response only, defaults to None
        :type name: str, optional
        :param payment_flow_type: payment_flow_type, defaults to None
        :type payment_flow_type: PaymentMethodTypePaymentFlowType, optional
        :param payment_options: payment_options, defaults to None
        :type payment_options: List[Field1], optional
        :param status: Indicates the status of the payment method. One of the following value is 1 means the payment_method_type is Valid, defaults to None
        :type status: str, optional
        :param supported_digital_wallet_providers: Describes the digital wallet providers that support the payment method. These providers may include **apple_pay** and **google_pay**., defaults to None
        :type supported_digital_wallet_providers: List[str], optional
        :param type_: Type of the payment method. For example, it_visa_card, defaults to None
        :type type_: str, optional
        :param virtual_payment_method_type: Indicates the name of the Web-based version of this payment method type, defaults to None
        :type virtual_payment_method_type: str, optional
        :param is_restricted: Indicates if the payment method type restricted or not., defaults to None
        :type is_restricted: bool, optional
        :param supports_subscription: Indicates if the payment method type supports subscription., defaults to None
        :type supports_subscription: bool, optional
        """
        self.amount_range_per_currency = self._define_list(
            amount_range_per_currency, PaymentAmountRangePerCurrencyInner
        )
        self.category = (
            self._enum_matching(category, Category.list(), "category")
            if category
            else None
        )
        self.country = self._define_str(
            "country",
            country,
            nullable=True,
            pattern="Name of the country. Two-letter ISO 3166-1 alpha-2 code.",
        )
        self.currencies = currencies
        self.fields = self._define_list(fields, Field1)
        self.image = self._define_str("image", image, nullable=True)
        self.is_cancelable = is_cancelable
        self.is_expirable = is_expirable
        self.is_online = is_online
        self.is_refundable = is_refundable
        self.is_tokenizable = is_tokenizable
        self.is_virtual = is_virtual
        self.maximum_expiration_seconds = self._define_number(
            "maximum_expiration_seconds", maximum_expiration_seconds, nullable=True
        )
        self.minimum_expiration_seconds = self._define_number(
            "minimum_expiration_seconds", minimum_expiration_seconds, nullable=True
        )
        self.multiple_overage_allowed = multiple_overage_allowed
        self.name = self._define_str("name", name, nullable=True)
        self.payment_flow_type = (
            self._enum_matching(
                payment_flow_type,
                PaymentMethodTypePaymentFlowType.list(),
                "payment_flow_type",
            )
            if payment_flow_type
            else None
        )
        self.payment_options = self._define_list(payment_options, Field1)
        self.status = self._define_str("status", status, nullable=True)
        self.supported_digital_wallet_providers = supported_digital_wallet_providers
        self.type_ = self._define_str("type_", type_, nullable=True)
        self.virtual_payment_method_type = self._define_str(
            "virtual_payment_method_type", virtual_payment_method_type, nullable=True
        )
        self.is_restricted = is_restricted
        self.supports_subscription = supports_subscription



from __future__ import annotations
from typing import Union
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.one_of_base_model import OneOfBaseModel
from .fee import Fee
from .payment import Payment
from .v1paymentssubscriptions_subscription_items import (
    V1paymentssubscriptionsSubscriptionItems,
)
from .payment_method_type import PaymentMethodType


class CheckoutSubscriptionsBodyPaymentMethodGuard(OneOfBaseModel):
    class_list = {"str": str, "PaymentMethodType": PaymentMethodType}


CheckoutSubscriptionsBodyPaymentMethod = Union[str, PaymentMethodType]


@JsonMap({})
class CheckoutSubscriptionsBody(BaseModel):
    """CheckoutSubscriptionsBody

    :param billing: Determines the method of billing. Set to **pay_automatically**.
    :type billing: str
    :param cancel_at_period_end: Determines the last date that charges accrue.\<BR\>* **true** - Charges accrue until the end of the current billing period, then the subscription is canceled. When no trial period is set, after this parameter is set to **true** the subscription will not be renewed at the next interval. When a trial period is set after this parameter is set to true, the subscription will not begin.\<BR\>* **false** - When the subscription is created, no end is defined. When Cancel Subscription is run, charges stop immediately and the subscription is canceled., defaults to None
    :type cancel_at_period_end: bool, optional
    :param complete_payment_url: URL where the customer is redirected when payment is successful, after returning from an external page such as a 3DS page. Does not support localhost URLs., defaults to None
    :type complete_payment_url: str, optional
    :param country: The two-letter ISO 3166-1 ALPHA-2 code for the country., defaults to None
    :type country: str, optional
    :param coupon: The ID of a coupon to apply a discount to the subscription. If the coupon defines a fixed monetary discount, it must use the same currency as the subscription. String starting with **coupon_**., defaults to None
    :type coupon: str, optional
    :param customer: ID of the customer who pays for this subscription. String starting with **cus_**.
    :type customer: str
    :param days_until_due: Number of days from the invoice date for customer to complete the payment., defaults to None
    :type days_until_due: float, optional
    :param error_payment_url: URL where the customer is redirected when payment is not successful, after returning from an external page, such as a 3DS page. Does not support localhost URLs., defaults to None
    :type error_payment_url: str, optional
    :param language: Determines the default language of the hosted page. For a list of values, see 'List Supported Languages'. \<BR\> * When this parameter is null, the language of the user's browser is used. \<BR\> * If the language of the user's browser cannot be determined, the default language is English., defaults to None
    :type language: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param merchant_main_button: Defines the text for the call-to-action button on the subscription checkout page.\<BR\> * When there is no trial period, the values are: **subscribe** (default), **purchase**, **pay** \<BR\> When there is a trial period, the value is **start_trial** (default)., defaults to None
    :type merchant_main_button: str, optional
    :param merchant_reference_id: Identifier defined by the client for reference purposes. Limit: 45 characters., defaults to None
    :type merchant_reference_id: str, optional
    :param page_expiration: End of the time when the customer can use the hosted page, in Unix time. If page_expiration is not set, the hosted page expires 14 days after creation.\<BR\> Range: 1 minute to 30 days., defaults to None
    :type page_expiration: float, optional
    :param payment_fees: payment_fees, defaults to None
    :type payment_fees: Fee, optional
    :param payment_fields: Collects money from a payment method and deposits it into one or more Rapyd Wallets, defaults to None
    :type payment_fields: Payment, optional
    :param payment_method: Payment Method object or ID., defaults to None
    :type payment_method: CheckoutSubscriptionsBodyPaymentMethod, optional
    :param subscription_items: Array of subscription items. Each item contains a plan (required) and a quantity
    :type subscription_items: List[V1paymentssubscriptionsSubscriptionItems]
    :param tax_percent: The percentage of tax that is applied to the entire amount of the invoice., defaults to None
    :type tax_percent: float, optional
    :param trial_end: The date and time of the end of the customer's free trial period, in Unix time, or the string **now**. Takes precedence over `trial_period_days`. If `trial_end` is not set by the client, Rapyd calculates this date by adding `trial_period_days` to the date in `created_at`. \<BR\> Relevant when `trial_period_days` is not defined., defaults to None
    :type trial_end: float, optional
    :param trial_from_plan: Determines whether a free trial period can be defined in a 'plan' object attached to the subscription.\<BR\> * **true** - Free trials can be defined in plans that are attached to the subscription. \<BR\> * **false** - Definitions of free trial in plans are ignored., defaults to None
    :type trial_from_plan: bool, optional
    :param trial_period_days: The number of days in the customer's free trial period. Integer. Range: 0-730. This value takes precedence over trial periods that are defined in a plan. Relevant when `trial_end` is not defined.The number of days in the customer's free trial period., defaults to None
    :type trial_period_days: float, optional
    """

    def __init__(
        self,
        billing: str,
        customer: str,
        subscription_items: List[V1paymentssubscriptionsSubscriptionItems],
        cancel_at_period_end: bool = None,
        complete_payment_url: str = None,
        country: str = None,
        coupon: str = None,
        days_until_due: float = None,
        error_payment_url: str = None,
        language: str = None,
        metadata: dict = None,
        merchant_main_button: str = None,
        merchant_reference_id: str = None,
        page_expiration: float = None,
        payment_fees: Fee = None,
        payment_fields: Payment = None,
        payment_method: CheckoutSubscriptionsBodyPaymentMethod = None,
        tax_percent: float = None,
        trial_end: float = None,
        trial_from_plan: bool = None,
        trial_period_days: float = None,
    ):
        """CheckoutSubscriptionsBody

        :param billing: Determines the method of billing. Set to **pay_automatically**.
        :type billing: str
        :param cancel_at_period_end: Determines the last date that charges accrue.\<BR\>* **true** - Charges accrue until the end of the current billing period, then the subscription is canceled. When no trial period is set, after this parameter is set to **true** the subscription will not be renewed at the next interval. When a trial period is set after this parameter is set to true, the subscription will not begin.\<BR\>* **false** - When the subscription is created, no end is defined. When Cancel Subscription is run, charges stop immediately and the subscription is canceled., defaults to None
        :type cancel_at_period_end: bool, optional
        :param complete_payment_url: URL where the customer is redirected when payment is successful, after returning from an external page such as a 3DS page. Does not support localhost URLs., defaults to None
        :type complete_payment_url: str, optional
        :param country: The two-letter ISO 3166-1 ALPHA-2 code for the country., defaults to None
        :type country: str, optional
        :param coupon: The ID of a coupon to apply a discount to the subscription. If the coupon defines a fixed monetary discount, it must use the same currency as the subscription. String starting with **coupon_**., defaults to None
        :type coupon: str, optional
        :param customer: ID of the customer who pays for this subscription. String starting with **cus_**.
        :type customer: str
        :param days_until_due: Number of days from the invoice date for customer to complete the payment., defaults to None
        :type days_until_due: float, optional
        :param error_payment_url: URL where the customer is redirected when payment is not successful, after returning from an external page, such as a 3DS page. Does not support localhost URLs., defaults to None
        :type error_payment_url: str, optional
        :param language: Determines the default language of the hosted page. For a list of values, see 'List Supported Languages'. \<BR\> * When this parameter is null, the language of the user's browser is used. \<BR\> * If the language of the user's browser cannot be determined, the default language is English., defaults to None
        :type language: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param merchant_main_button: Defines the text for the call-to-action button on the subscription checkout page.\<BR\> * When there is no trial period, the values are: **subscribe** (default), **purchase**, **pay** \<BR\> When there is a trial period, the value is **start_trial** (default)., defaults to None
        :type merchant_main_button: str, optional
        :param merchant_reference_id: Identifier defined by the client for reference purposes. Limit: 45 characters., defaults to None
        :type merchant_reference_id: str, optional
        :param page_expiration: End of the time when the customer can use the hosted page, in Unix time. If page_expiration is not set, the hosted page expires 14 days after creation.\<BR\> Range: 1 minute to 30 days., defaults to None
        :type page_expiration: float, optional
        :param payment_fees: payment_fees, defaults to None
        :type payment_fees: Fee, optional
        :param payment_fields: Collects money from a payment method and deposits it into one or more Rapyd Wallets, defaults to None
        :type payment_fields: Payment, optional
        :param payment_method: Payment Method object or ID., defaults to None
        :type payment_method: CheckoutSubscriptionsBodyPaymentMethod, optional
        :param subscription_items: Array of subscription items. Each item contains a plan (required) and a quantity
        :type subscription_items: List[V1paymentssubscriptionsSubscriptionItems]
        :param tax_percent: The percentage of tax that is applied to the entire amount of the invoice., defaults to None
        :type tax_percent: float, optional
        :param trial_end: The date and time of the end of the customer's free trial period, in Unix time, or the string **now**. Takes precedence over `trial_period_days`. If `trial_end` is not set by the client, Rapyd calculates this date by adding `trial_period_days` to the date in `created_at`. \<BR\> Relevant when `trial_period_days` is not defined., defaults to None
        :type trial_end: float, optional
        :param trial_from_plan: Determines whether a free trial period can be defined in a 'plan' object attached to the subscription.\<BR\> * **true** - Free trials can be defined in plans that are attached to the subscription. \<BR\> * **false** - Definitions of free trial in plans are ignored., defaults to None
        :type trial_from_plan: bool, optional
        :param trial_period_days: The number of days in the customer's free trial period. Integer. Range: 0-730. This value takes precedence over trial periods that are defined in a plan. Relevant when `trial_end` is not defined.The number of days in the customer's free trial period., defaults to None
        :type trial_period_days: float, optional
        """
        self.billing = billing
        self.cancel_at_period_end = cancel_at_period_end
        self.complete_payment_url = self._define_str(
            "complete_payment_url", complete_payment_url, nullable=True
        )
        self.country = self._define_str("country", country, nullable=True)
        self.coupon = self._define_str("coupon", coupon, nullable=True)
        self.customer = customer
        self.days_until_due = self._define_number(
            "days_until_due", days_until_due, nullable=True
        )
        self.error_payment_url = self._define_str(
            "error_payment_url", error_payment_url, nullable=True
        )
        self.language = self._define_str("language", language, nullable=True)
        self.metadata = metadata
        self.merchant_main_button = self._define_str(
            "merchant_main_button", merchant_main_button, nullable=True
        )
        self.merchant_reference_id = self._define_str(
            "merchant_reference_id", merchant_reference_id, nullable=True
        )
        self.page_expiration = self._define_number(
            "page_expiration", page_expiration, nullable=True
        )
        self.payment_fees = self._define_object(payment_fees, Fee)
        self.payment_fields = self._define_object(payment_fields, Payment)
        self.payment_method = CheckoutSubscriptionsBodyPaymentMethodGuard.return_one_of(
            payment_method
        )
        self.subscription_items = self._define_list(
            subscription_items, V1paymentssubscriptionsSubscriptionItems
        )
        self.tax_percent = self._define_number(
            "tax_percent", tax_percent, nullable=True
        )
        self.trial_end = self._define_number("trial_end", trial_end, nullable=True)
        self.trial_from_plan = trial_from_plan
        self.trial_period_days = self._define_number(
            "trial_period_days", trial_period_days, nullable=True
        )



from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


class CardTransactionPosEntryMode(Enum):
    """An enumeration representing different categories.

    :cvar MAGSTRIPE: "magstripe"
    :vartype MAGSTRIPE: str
    :cvar MANUALENTERED: "manual_entered"
    :vartype MANUALENTERED: str
    :cvar EMV: "emv"
    :vartype EMV: str
    :cvar EMVSTANDIN: "emv_standin"
    :vartype EMVSTANDIN: str
    :cvar NFC: "nfc"
    :vartype NFC: str
    :cvar NETWORKTOKEN: "network_token"
    :vartype NETWORKTOKEN: str
    :cvar ECOMMERCE: "ecommerce"
    :vartype ECOMMERCE: str
    :cvar _3DSECOMMERCE: "3ds_ecommerce"
    :vartype _3DSECOMMERCE: str
    :cvar ADJUSTMENT: "adjustment"
    :vartype ADJUSTMENT: str
    """

    MAGSTRIPE = "magstripe"
    MANUALENTERED = "manual_entered"
    EMV = "emv"
    EMVSTANDIN = "emv_standin"
    NFC = "nfc"
    NETWORKTOKEN = "network_token"
    ECOMMERCE = "ecommerce"
    _3DSECOMMERCE = "3ds_ecommerce"
    ADJUSTMENT = "adjustment"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, CardTransactionPosEntryMode._member_map_.values())
        )


@JsonMap({"id_": "id"})
class CardTransaction(BaseModel):
    """CardTransaction

    :param amount: Amount of the transaction, in units defined in currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015. Response only., defaults to None
    :type amount: float, optional
    :param auth_code: Authorization code sent to the merchant. Relevant to authorizations approved by Rapyd. Response only., defaults to None
    :type auth_code: str, optional
    :param authorization_approved_by: Approver of the transaction when remote authorization is used. See Remote Authorization., defaults to None
    :type authorization_approved_by: str, optional
    :param bin: The first 6 digits of the card number. Response only., defaults to None
    :type bin: str, optional
    :param card_authorization: card_authorization, defaults to None
    :type card_authorization: str, optional
    :param card_id: ID of the card. String starting with card_. Response only., defaults to None
    :type card_id: str, optional
    :param card_program: ID of the card program that the card is issued from. String starting with **cardprog_**. Response only., defaults to None
    :type card_program: str, optional
    :param created_at: Time of creation of the transaction object, in Unix time. Response only., defaults to None
    :type created_at: float, optional
    :param currency: currency, defaults to None
    :type currency: str, optional
    :param fx_rate: The exchange rate. Response only., defaults to None
    :type fx_rate: float, optional
    :param id_: ID of the issued card transaction object. String starting with **cit_**., defaults to None
    :type id_: str, optional
    :param issuing_txn_type: Type of the transaction., defaults to None
    :type issuing_txn_type: str, optional
    :param last4: The last 4 digits of the card number. Response only., defaults to None
    :type last4: str, optional
    :param merchant_category_code: Four-digit merchant category code (MCC) of the initiator of the transaction, as defined in ISO 18245. Response only., defaults to None
    :type merchant_category_code: str, optional
    :param merchant_name_location: Name and location of the merchant. Maximum 40 characters. Response only., defaults to None
    :type merchant_name_location: str, optional
    :param original_transaction_id: ID of the original card transaction. Response only., defaults to None
    :type original_transaction_id: str, optional
    :param original_txn_amount: Original amount for FX transactions, when currency is different from original_txn_currency. Response only., defaults to None
    :type original_txn_amount: float, optional
    :param original_txn_currency: Original currency in FX transaction. Three-letter ISO 4217 code. Response only., defaults to None
    :type original_txn_currency: str, optional
    :param pos_entry_mode: The mode of the POS entry. One of the following, defaults to None
    :type pos_entry_mode: CardTransactionPosEntryMode, optional
    :param retrieval_reference_number: Reserved., defaults to None
    :type retrieval_reference_number: str, optional
    :param systems_trace_audit_number: Reserved, defaults to None
    :type systems_trace_audit_number: str, optional
    :param wallet_transaction_id: ID of the wallet transaction. String starting with **wt_**. Response only., defaults to None
    :type wallet_transaction_id: str, optional
    """

    def __init__(
        self,
        amount: float = None,
        auth_code: str = None,
        authorization_approved_by: str = None,
        bin: str = None,
        card_authorization: str = None,
        card_id: str = None,
        card_program: str = None,
        created_at: float = None,
        currency: str = None,
        fx_rate: float = None,
        id_: str = None,
        issuing_txn_type: str = None,
        last4: str = None,
        merchant_category_code: str = None,
        merchant_name_location: str = None,
        original_transaction_id: str = None,
        original_txn_amount: float = None,
        original_txn_currency: str = None,
        pos_entry_mode: CardTransactionPosEntryMode = None,
        retrieval_reference_number: str = None,
        systems_trace_audit_number: str = None,
        wallet_transaction_id: str = None,
    ):
        """CardTransaction

        :param amount: Amount of the transaction, in units defined in currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015. Response only., defaults to None
        :type amount: float, optional
        :param auth_code: Authorization code sent to the merchant. Relevant to authorizations approved by Rapyd. Response only., defaults to None
        :type auth_code: str, optional
        :param authorization_approved_by: Approver of the transaction when remote authorization is used. See Remote Authorization., defaults to None
        :type authorization_approved_by: str, optional
        :param bin: The first 6 digits of the card number. Response only., defaults to None
        :type bin: str, optional
        :param card_authorization: card_authorization, defaults to None
        :type card_authorization: str, optional
        :param card_id: ID of the card. String starting with card_. Response only., defaults to None
        :type card_id: str, optional
        :param card_program: ID of the card program that the card is issued from. String starting with **cardprog_**. Response only., defaults to None
        :type card_program: str, optional
        :param created_at: Time of creation of the transaction object, in Unix time. Response only., defaults to None
        :type created_at: float, optional
        :param currency: currency, defaults to None
        :type currency: str, optional
        :param fx_rate: The exchange rate. Response only., defaults to None
        :type fx_rate: float, optional
        :param id_: ID of the issued card transaction object. String starting with **cit_**., defaults to None
        :type id_: str, optional
        :param issuing_txn_type: Type of the transaction., defaults to None
        :type issuing_txn_type: str, optional
        :param last4: The last 4 digits of the card number. Response only., defaults to None
        :type last4: str, optional
        :param merchant_category_code: Four-digit merchant category code (MCC) of the initiator of the transaction, as defined in ISO 18245. Response only., defaults to None
        :type merchant_category_code: str, optional
        :param merchant_name_location: Name and location of the merchant. Maximum 40 characters. Response only., defaults to None
        :type merchant_name_location: str, optional
        :param original_transaction_id: ID of the original card transaction. Response only., defaults to None
        :type original_transaction_id: str, optional
        :param original_txn_amount: Original amount for FX transactions, when currency is different from original_txn_currency. Response only., defaults to None
        :type original_txn_amount: float, optional
        :param original_txn_currency: Original currency in FX transaction. Three-letter ISO 4217 code. Response only., defaults to None
        :type original_txn_currency: str, optional
        :param pos_entry_mode: The mode of the POS entry. One of the following, defaults to None
        :type pos_entry_mode: CardTransactionPosEntryMode, optional
        :param retrieval_reference_number: Reserved., defaults to None
        :type retrieval_reference_number: str, optional
        :param systems_trace_audit_number: Reserved, defaults to None
        :type systems_trace_audit_number: str, optional
        :param wallet_transaction_id: ID of the wallet transaction. String starting with **wt_**. Response only., defaults to None
        :type wallet_transaction_id: str, optional
        """
        self.amount = self._define_number("amount", amount, nullable=True)
        self.auth_code = self._define_str("auth_code", auth_code, nullable=True)
        self.authorization_approved_by = self._define_str(
            "authorization_approved_by", authorization_approved_by, nullable=True
        )
        self.bin = self._define_str("bin", bin, nullable=True)
        self.card_authorization = self._define_str(
            "card_authorization", card_authorization, nullable=True
        )
        self.card_id = self._define_str("card_id", card_id, nullable=True)
        self.card_program = self._define_str(
            "card_program", card_program, nullable=True
        )
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.currency = self._define_str(
            "currency",
            currency,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.fx_rate = self._define_number("fx_rate", fx_rate, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.issuing_txn_type = self._define_str(
            "issuing_txn_type", issuing_txn_type, nullable=True
        )
        self.last4 = self._define_str("last4", last4, nullable=True)
        self.merchant_category_code = self._define_str(
            "merchant_category_code", merchant_category_code, nullable=True
        )
        self.merchant_name_location = self._define_str(
            "merchant_name_location", merchant_name_location, nullable=True
        )
        self.original_transaction_id = self._define_str(
            "original_transaction_id", original_transaction_id, nullable=True
        )
        self.original_txn_amount = self._define_number(
            "original_txn_amount", original_txn_amount, nullable=True
        )
        self.original_txn_currency = self._define_str(
            "original_txn_currency", original_txn_currency, nullable=True
        )
        self.pos_entry_mode = (
            self._enum_matching(
                pos_entry_mode, CardTransactionPosEntryMode.list(), "pos_entry_mode"
            )
            if pos_entry_mode
            else None
        )
        self.retrieval_reference_number = self._define_str(
            "retrieval_reference_number", retrieval_reference_number, nullable=True
        )
        self.systems_trace_audit_number = self._define_str(
            "systems_trace_audit_number", systems_trace_audit_number, nullable=True
        )
        self.wallet_transaction_id = self._define_str(
            "wallet_transaction_id", wallet_transaction_id, nullable=True
        )



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .escrow_ewallets import EscrowEwallets


@JsonMap({})
class EscrowEscrowReleasesBody(BaseModel):
    """EscrowEscrowReleasesBody

    :param ewallets: Describes the wallets and the releases from escrow.
    :type ewallets: EscrowEwallets
    """

    def __init__(self, ewallets: EscrowEwallets):
        """EscrowEscrowReleasesBody

        :param ewallets: Describes the wallets and the releases from escrow.
        :type ewallets: EscrowEwallets
        """
        self.ewallets = self._define_object(ewallets, EscrowEwallets)



from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .v1products_package_dimensions import V1productsPackageDimensions


@JsonMap({})
class SubscriptionProductUpdateProductRequest2(BaseModel):
    """SubscriptionProductUpdateProductRequest2

    :param active: Indicates whether the product is currently available for purchase. Default is true., defaults to None
    :type active: bool, optional
    :param attributes: Array of up to 5 alphanumeric strings defined by the merchant. Each string defines the key in a key-value pair in the 'attributes' object in the corresponding 'sku' objects., defaults to None
    :type attributes: List[str], optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param name: The name of the product or service that is displayed to the customer.
    :type name: str
    :param package_dimensions: Describes the physical size and weight of the product. Relevant when type is goods., defaults to None
    :type package_dimensions: V1productsPackageDimensions, optional
    :param statement_descriptor: A text description that appears in the customer's invoice. Limited to 22 characters. Relevant when type is service., defaults to None
    :type statement_descriptor: str, optional
    :param unit_label: A label that represents units of this product, such as seats, on customersâ€™, defaults to None
    :type unit_label: str, optional
    """

    def __init__(
        self,
        name: str,
        active: bool = None,
        attributes: List[str] = None,
        metadata: dict = None,
        package_dimensions: V1productsPackageDimensions = None,
        statement_descriptor: str = None,
        unit_label: str = None,
    ):
        """SubscriptionProductUpdateProductRequest2

        :param active: Indicates whether the product is currently available for purchase. Default is true., defaults to None
        :type active: bool, optional
        :param attributes: Array of up to 5 alphanumeric strings defined by the merchant. Each string defines the key in a key-value pair in the 'attributes' object in the corresponding 'sku' objects., defaults to None
        :type attributes: List[str], optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param name: The name of the product or service that is displayed to the customer.
        :type name: str
        :param package_dimensions: Describes the physical size and weight of the product. Relevant when type is goods., defaults to None
        :type package_dimensions: V1productsPackageDimensions, optional
        :param statement_descriptor: A text description that appears in the customer's invoice. Limited to 22 characters. Relevant when type is service., defaults to None
        :type statement_descriptor: str, optional
        :param unit_label: A label that represents units of this product, such as seats, on customersâ€™, defaults to None
        :type unit_label: str, optional
        """
        self.active = active
        self.attributes = attributes
        self.metadata = metadata
        self.name = name
        self.package_dimensions = self._define_object(
            package_dimensions, V1productsPackageDimensions
        )
        self.statement_descriptor = self._define_str(
            "statement_descriptor", statement_descriptor, nullable=True
        )
        self.unit_label = self._define_str("unit_label", unit_label, nullable=True)



from typing import List
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import InlineResponse200_48, InlineResponse200_49, OrderIdReturnsBody


class OrderReturnService(BaseService):

    @cast_models
    def returns_order(
        self,
        request_body: OrderIdReturnsBody,
        order_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_48:
        """Make a return against an order. The maximum amount of the return is the amount of the order

        :param request_body: The request body.
        :type request_body: OrderIdReturnsBody
        :param order_id: ID of the order. String starting with order_.
        :type order_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: SUCCESS
        :rtype: InlineResponse200_48
        """

        Validator(OrderIdReturnsBody).validate(request_body)
        Validator(str).validate(order_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/orders/{{orderId}}/returns",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("orderId", order_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_48._unmap(response)

    @cast_models
    def list_order_return(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        limit: str = None,
        ending_before: str = None,
        starting_after: str = None,
        tokens: List[str] = None,
        idempotency: str = None,
    ) -> InlineResponse200_49:
        """Retrieve a list of all order returns.

        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param limit: The maximum number of returns to list. Range - 1-100. Default is 10., defaults to None
        :type limit: str, optional
        :param ending_before: The ID of the order created after the last order you want to retrieve a return from., defaults to None
        :type ending_before: str, optional
        :param starting_after: The ID of the order created before the first order you want to retrieve a return from., defaults to None
        :type starting_after: str, optional
        :param tokens: Filters the list for orders related to the specified order., defaults to None
        :type tokens: List[str], optional
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List of orders.
        :rtype: InlineResponse200_49
        """

        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(limit)
        Validator(str).is_optional().validate(ending_before)
        Validator(str).is_optional().validate(starting_after)
        Validator(str).is_array().is_optional().validate(tokens)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/order_returns", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_query("limit", limit, explode=False)
            .add_query("ending_before", ending_before, explode=False)
            .add_query("starting_after", starting_after, explode=False)
            .add_query("tokens", tokens, explode=False)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_49._unmap(response)

    @cast_models
    def retrieve_order_return(
        self,
        order_returns_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_48:
        """Retrieve the details of a return.

        :param order_returns_id: ID of the return. String starting with **orre_**.
        :type order_returns_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: SUCCESS
        :rtype: InlineResponse200_48
        """

        Validator(str).validate(order_returns_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/order_returns/{{orderReturnsId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("orderReturnsId", order_returns_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_48._unmap(response)



from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .field_1 import Field1


@JsonMap({"type_": "type"})
class PaymentMethodTypeRequiredFields(BaseModel):
    """Payment Method Type required fields - this is the response of GET required fields for Payment_Method_Type

    :param fields: Payment Method Type required field, defaults to None
    :type fields: List[Field1], optional
    :param maximum_expiration_seconds: The maximum time (in seconds) that the merchant can set for completing the payment. Relevant when is_expirable is true, defaults to None
    :type maximum_expiration_seconds: float, optional
    :param minimum_expiration_seconds: The minimum time (in seconds) that the merchant can set for completing the payment. Relevant when is_expirable is true, defaults to None
    :type minimum_expiration_seconds: float, optional
    :param payment_method_options: payment method option, defaults to None
    :type payment_method_options: List[Field1], optional
    :param payment_options: payment_options, defaults to None
    :type payment_options: List[Field1], optional
    :param type_: type_, defaults to None
    :type type_: str, optional
    """

    def __init__(
        self,
        fields: List[Field1] = None,
        maximum_expiration_seconds: float = None,
        minimum_expiration_seconds: float = None,
        payment_method_options: List[Field1] = None,
        payment_options: List[Field1] = None,
        type_: str = None,
    ):
        """Payment Method Type required fields - this is the response of GET required fields for Payment_Method_Type

        :param fields: Payment Method Type required field, defaults to None
        :type fields: List[Field1], optional
        :param maximum_expiration_seconds: The maximum time (in seconds) that the merchant can set for completing the payment. Relevant when is_expirable is true, defaults to None
        :type maximum_expiration_seconds: float, optional
        :param minimum_expiration_seconds: The minimum time (in seconds) that the merchant can set for completing the payment. Relevant when is_expirable is true, defaults to None
        :type minimum_expiration_seconds: float, optional
        :param payment_method_options: payment method option, defaults to None
        :type payment_method_options: List[Field1], optional
        :param payment_options: payment_options, defaults to None
        :type payment_options: List[Field1], optional
        :param type_: type_, defaults to None
        :type type_: str, optional
        """
        self.fields = self._define_list(fields, Field1)
        self.maximum_expiration_seconds = self._define_number(
            "maximum_expiration_seconds", maximum_expiration_seconds, nullable=True
        )
        self.minimum_expiration_seconds = self._define_number(
            "minimum_expiration_seconds", minimum_expiration_seconds, nullable=True
        )
        self.payment_method_options = self._define_list(payment_method_options, Field1)
        self.payment_options = self._define_list(payment_options, Field1)
        self.type_ = self._define_str("type_", type_, nullable=True)



from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .ewallet_transaction import EwalletTransaction
from .status import Status


@JsonMap({})
class InlineResponse200_101(BaseModel):
    """InlineResponse200_101

    :param data: data, defaults to None
    :type data: List[EwalletTransaction], optional
    :param status: status, defaults to None
    :type status: Status, optional
    """

    def __init__(self, data: List[EwalletTransaction] = None, status: Status = None):
        """InlineResponse200_101

        :param data: data, defaults to None
        :type data: List[EwalletTransaction], optional
        :param status: status, defaults to None
        :type status: Status, optional
        """
        self.data = self._define_list(data, EwalletTransaction)
        self.status = self._define_object(status, Status)



from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .contact import Contact


@JsonMap({})
class EwalletContacts(BaseModel):
    """EwalletContacts

    :param data: data, defaults to None
    :type data: List[Contact], optional
    :param has_more: has_more, defaults to None
    :type has_more: bool, optional
    :param total_count: total_count, defaults to None
    :type total_count: int, optional
    :param url: url, defaults to None
    :type url: str, optional
    """

    def __init__(
        self,
        data: List[Contact] = None,
        has_more: bool = None,
        total_count: int = None,
        url: str = None,
    ):
        """EwalletContacts

        :param data: data, defaults to None
        :type data: List[Contact], optional
        :param has_more: has_more, defaults to None
        :type has_more: bool, optional
        :param total_count: total_count, defaults to None
        :type total_count: int, optional
        :param url: url, defaults to None
        :type url: str, optional
        """
        self.data = self._define_list(data, Contact)
        self.has_more = has_more
        self.total_count = self._define_number(
            "total_count", total_count, nullable=True
        )
        self.url = self._define_str("url", url, nullable=True)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class CardsAdjustmentBody(BaseModel):
    """CardsAdjustmentBody

    :param amount: The amount of the adjustment, in units of the `currency` defined in currency. Decimal.
    :type amount: float
    :param authorization_id: ID of the authorization. String starting with **cardauth_**. Use the value of `id` in the response to 'Simulate a Card Transaction Authorization Request - EEA'.
    :type authorization_id: str
    :param card_id: ID of the card. String starting with **card_**.
    :type card_id: str
    :param currency: The adjustment currency. Three-letter ISO 4217 code.
    :type currency: str
    """

    def __init__(
        self, amount: float, authorization_id: str, card_id: str, currency: str
    ):
        """CardsAdjustmentBody

        :param amount: The amount of the adjustment, in units of the `currency` defined in currency. Decimal.
        :type amount: float
        :param authorization_id: ID of the authorization. String starting with **cardauth_**. Use the value of `id` in the response to 'Simulate a Card Transaction Authorization Request - EEA'.
        :type authorization_id: str
        :param card_id: ID of the card. String starting with **card_**.
        :type card_id: str
        :param currency: The adjustment currency. Three-letter ISO 4217 code.
        :type currency: str
        """
        self.amount = amount
        self.authorization_id = authorization_id
        self.card_id = card_id
        self.currency = currency



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class InvoiceItemsInvoiceItemBody(BaseModel):
    """InvoiceItemsInvoiceItemBody

    :param days_until_due: Number of days the customer has for paying this invoice. Integer., defaults to None
    :type days_until_due: float, optional
    :param description: Description of the invoice., defaults to None
    :type description: str, optional
    :param due_date: The date payment is due on this invoice. This value is calculated from the date the invoice is created, plus the number of days specified in the `days_until_due` field. Relevant when `billing` is **send_invoice**. Format is in Unix time., defaults to None
    :type due_date: float, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param payment_fields: Additional `payment_options`., defaults to None
    :type payment_fields: dict, optional
    :param statement_descriptor: Description of the invoice for the customer's credit card statement. Limited to 22 characters., defaults to None
    :type statement_descriptor: str, optional
    :param tax_percent: The percentage tax rate that is applied to the subtotal of the invoice, after subtracting all discounts. Decimal, up to four decimal places. Range: 0-100, defaults to None
    :type tax_percent: float, optional
    """

    def __init__(
        self,
        days_until_due: float = None,
        description: str = None,
        due_date: float = None,
        metadata: dict = None,
        payment_fields: dict = None,
        statement_descriptor: str = None,
        tax_percent: float = None,
    ):
        """InvoiceItemsInvoiceItemBody

        :param days_until_due: Number of days the customer has for paying this invoice. Integer., defaults to None
        :type days_until_due: float, optional
        :param description: Description of the invoice., defaults to None
        :type description: str, optional
        :param due_date: The date payment is due on this invoice. This value is calculated from the date the invoice is created, plus the number of days specified in the `days_until_due` field. Relevant when `billing` is **send_invoice**. Format is in Unix time., defaults to None
        :type due_date: float, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param payment_fields: Additional `payment_options`., defaults to None
        :type payment_fields: dict, optional
        :param statement_descriptor: Description of the invoice for the customer's credit card statement. Limited to 22 characters., defaults to None
        :type statement_descriptor: str, optional
        :param tax_percent: The percentage tax rate that is applied to the subtotal of the invoice, after subtracting all discounts. Decimal, up to four decimal places. Range: 0-100, defaults to None
        :type tax_percent: float, optional
        """
        self.days_until_due = self._define_number(
            "days_until_due", days_until_due, nullable=True
        )
        self.description = self._define_str("description", description, nullable=True)
        self.due_date = self._define_number("due_date", due_date, nullable=True)
        self.metadata = metadata
        self.payment_fields = payment_fields
        self.statement_descriptor = self._define_str(
            "statement_descriptor", statement_descriptor, nullable=True
        )
        self.tax_percent = self._define_number(
            "tax_percent", tax_percent, nullable=True
        )



from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


class HostedIdvBodyRequestType(Enum):
    """An enumeration representing different categories.

    :cvar STORE: "store"
    :vartype STORE: str
    :cvar VERIFY: "verify"
    :vartype VERIFY: str
    """

    STORE = "store"
    VERIFY = "verify"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, HostedIdvBodyRequestType._member_map_.values())
        )


@JsonMap({})
class HostedIdvBody(BaseModel):
    """HostedIdvBody

    :param cancel_url: URL where the customer is redirected after pressing **Back to Website** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
    :type cancel_url: str, optional
    :param complete_url: URL where the customer is redirected after pressing **Close** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
    :type complete_url: str, optional
    :param contact: ID of the wallet contact. String starting with **cont_**.
    :type contact: str
    :param country: The two-letter ISO 3166-1 ALPHA-2 code for the country of the identification document. Must match the wallet contact country.
    :type country: str
    :param document_type: Type of the identification document. Two-letter code.\<BR\>See also 'List Official Identification Documents'., defaults to None
    :type document_type: str, optional
    :param ewallet: ID of the Rapyd Wallet. String starting with **ewallet_**.
    :type ewallet: str
    :param force_camera: Requires the applicant to use the device's camera for a current face image, and prohibits uploading an existing image file. Default is **false**., defaults to None
    :type force_camera: bool, optional
    :param language: Determines the default language of the hosted page. For a list of values, see [List Supported Languages](https://docs.rapyd.net/en/list-supported-languages.html). \<BR\> * When this parameter is null, the language of the user's browser is used. \<BR\> * If the language of the user's browser cannot be determined, the default language is English., defaults to None
    :type language: str, optional
    :param page_expiration: End of the time when the customer can use the hosted page, in Unix time. If `page_expiration` is not set, the hosted page expires 14 days after creation.\<BR\>**Range**: 1 minute to 30 days., defaults to None
    :type page_expiration: float, optional
    :param reference_id: ID of the identity verification request. Must be unique for each request. Defined by the client.
    :type reference_id: str
    :param request_type: Determines the action that is taken on the request. One of the following:\<BR\>* **store** - Store the images on the Rapyd platform.\<BR\>* **verify** - Verify the identity of the person in the images., defaults to None
    :type request_type: HostedIdvBodyRequestType, optional
    """

    def __init__(
        self,
        contact: str,
        country: str,
        ewallet: str,
        reference_id: str,
        cancel_url: str = None,
        complete_url: str = None,
        document_type: str = None,
        force_camera: bool = None,
        language: str = None,
        page_expiration: float = None,
        request_type: HostedIdvBodyRequestType = None,
    ):
        """HostedIdvBody

        :param cancel_url: URL where the customer is redirected after pressing **Back to Website** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
        :type cancel_url: str, optional
        :param complete_url: URL where the customer is redirected after pressing **Close** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
        :type complete_url: str, optional
        :param contact: ID of the wallet contact. String starting with **cont_**.
        :type contact: str
        :param country: The two-letter ISO 3166-1 ALPHA-2 code for the country of the identification document. Must match the wallet contact country.
        :type country: str
        :param document_type: Type of the identification document. Two-letter code.\<BR\>See also 'List Official Identification Documents'., defaults to None
        :type document_type: str, optional
        :param ewallet: ID of the Rapyd Wallet. String starting with **ewallet_**.
        :type ewallet: str
        :param force_camera: Requires the applicant to use the device's camera for a current face image, and prohibits uploading an existing image file. Default is **false**., defaults to None
        :type force_camera: bool, optional
        :param language: Determines the default language of the hosted page. For a list of values, see [List Supported Languages](https://docs.rapyd.net/en/list-supported-languages.html). \<BR\> * When this parameter is null, the language of the user's browser is used. \<BR\> * If the language of the user's browser cannot be determined, the default language is English., defaults to None
        :type language: str, optional
        :param page_expiration: End of the time when the customer can use the hosted page, in Unix time. If `page_expiration` is not set, the hosted page expires 14 days after creation.\<BR\>**Range**: 1 minute to 30 days., defaults to None
        :type page_expiration: float, optional
        :param reference_id: ID of the identity verification request. Must be unique for each request. Defined by the client.
        :type reference_id: str
        :param request_type: Determines the action that is taken on the request. One of the following:\<BR\>* **store** - Store the images on the Rapyd platform.\<BR\>* **verify** - Verify the identity of the person in the images., defaults to None
        :type request_type: HostedIdvBodyRequestType, optional
        """
        self.cancel_url = self._define_str("cancel_url", cancel_url, nullable=True)
        self.complete_url = self._define_str(
            "complete_url", complete_url, nullable=True
        )
        self.contact = contact
        self.country = country
        self.document_type = self._define_str(
            "document_type", document_type, nullable=True
        )
        self.ewallet = ewallet
        self.force_camera = force_camera
        self.language = self._define_str("language", language, nullable=True)
        self.page_expiration = self._define_number(
            "page_expiration", page_expiration, nullable=True
        )
        self.reference_id = reference_id
        self.request_type = (
            self._enum_matching(
                request_type, HostedIdvBodyRequestType.list(), "request_type"
            )
            if request_type
            else None
        )



from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .simulate_card_transaction_authorization_request_eea_auth_response import (
    SimulateCardTransactionAuthorizationRequestEeaAuthResponse,
)


class IssuingTxnType(Enum):
    """An enumeration representing different categories.

    :cvar ADJUSTMENT: "ADJUSTMENT"
    :vartype ADJUSTMENT: str
    :cvar ATMFEE: "ATM_FEE"
    :vartype ATMFEE: str
    :cvar CREDIT: "CREDIT"
    :vartype CREDIT: str
    :cvar REFUND: "REFUND"
    :vartype REFUND: str
    :cvar REVERSAL: "REVERSAL"
    :vartype REVERSAL: str
    :cvar SALE: "SALE"
    :vartype SALE: str
    """

    ADJUSTMENT = "ADJUSTMENT"
    ATMFEE = "ATM_FEE"
    CREDIT = "CREDIT"
    REFUND = "REFUND"
    REVERSAL = "REVERSAL"
    SALE = "SALE"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, IssuingTxnType._member_map_.values()))


class SimulateCardTransactionAuthorizationRequestEeaPosEntryMode(Enum):
    """An enumeration representing different categories.

    :cvar ADJUSTMENT: "adjustment"
    :vartype ADJUSTMENT: str
    :cvar ECOMMERCE: "ecommerce"
    :vartype ECOMMERCE: str
    :cvar EMV: "emv"
    :vartype EMV: str
    :cvar EMVSTANDIN: "emv_standin"
    :vartype EMVSTANDIN: str
    :cvar MAGSTRIPE: "magstripe"
    :vartype MAGSTRIPE: str
    :cvar MANUALENTERED: "manual_entered"
    :vartype MANUALENTERED: str
    :cvar NETWORKTOKEN: "network_token"
    :vartype NETWORKTOKEN: str
    :cvar NFC: "nfc"
    :vartype NFC: str
    :cvar _3DSECOMMERCE: "3ds_ecommerce"
    :vartype _3DSECOMMERCE: str
    """

    ADJUSTMENT = "adjustment"
    ECOMMERCE = "ecommerce"
    EMV = "emv"
    EMVSTANDIN = "emv_standin"
    MAGSTRIPE = "magstripe"
    MANUALENTERED = "manual_entered"
    NETWORKTOKEN = "network_token"
    NFC = "nfc"
    _3DSECOMMERCE = "3ds_ecommerce"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                SimulateCardTransactionAuthorizationRequestEeaPosEntryMode._member_map_.values(),
            )
        )


@JsonMap({"id_": "id"})
class SimulateCardTransactionAuthorizationRequestEea(BaseModel):
    """SimulateCardTransactionAuthorizationRequestEea

    :param amount: Amount of the transaction, in units defined in `currency`., defaults to None
    :type amount: float, optional
    :param auth_response: Response related to an authorization. Contains the following fields:, defaults to None
    :type auth_response: SimulateCardTransactionAuthorizationRequestEeaAuthResponse, optional
    :param card_id: ID of the card. String starting with **card_**., defaults to None
    :type card_id: str, optional
    :param card_transaction_id: ID of the transaction. String starting with **cit_**., defaults to None
    :type card_transaction_id: str, optional
    :param currency_code: Currency of the transaction. Three-letter ISO 4217 code.Currency of the refund transaction. Three-letter ISO 4217 code., defaults to None
    :type currency_code: str, optional
    :param fx_rate: The exchange rate. Relevant to capture (clearing) transactions., defaults to None
    :type fx_rate: str, optional
    :param id_: ID of the issued card transaction. String starting with **cit_**., defaults to None
    :type id_: str, optional
    :param issuing_txn_type: Type of transaction on the issued card., defaults to None
    :type issuing_txn_type: IssuingTxnType, optional
    :param last4: Last 4 digits of the card number., defaults to None
    :type last4: str, optional
    :param merchant_category_code: Four-digit merchant category code (MCC) of the initiator of the transaction, as defined in ISO 18245., defaults to None
    :type merchant_category_code: str, optional
    :param merchant_name_location: Name and location of the merchant., defaults to None
    :type merchant_name_location: str, optional
    :param original_transaction_id: ID of the original issued card transaction. String starting with **cit_**., defaults to None
    :type original_transaction_id: str, optional
    :param original_txn_amount: Original amount for FX transactions, when `currency` is different from `original_txn_currency`., defaults to None
    :type original_txn_amount: float, optional
    :param original_txn_currency: Original currency in an FX transaction., defaults to None
    :type original_txn_currency: str, optional
    :param pos_entry_mode: The mode of entry of the transaction at the point of sale., defaults to None
    :type pos_entry_mode: SimulateCardTransactionAuthorizationRequestEeaPosEntryMode, optional
    :param retrieval_reference_number: Retrieval reference number for the card transaction., defaults to None
    :type retrieval_reference_number: str, optional
    :param systems_trace_audit_number: Reserved., defaults to None
    :type systems_trace_audit_number: str, optional
    :param wallet_transaction_id: ID of the wallet transaction String starting with **wt_**., defaults to None
    :type wallet_transaction_id: str, optional
    """

    def __init__(
        self,
        amount: float = None,
        auth_response: SimulateCardTransactionAuthorizationRequestEeaAuthResponse = None,
        card_id: str = None,
        card_transaction_id: str = None,
        currency_code: str = None,
        fx_rate: str = None,
        id_: str = None,
        issuing_txn_type: IssuingTxnType = None,
        last4: str = None,
        merchant_category_code: str = None,
        merchant_name_location: str = None,
        original_transaction_id: str = None,
        original_txn_amount: float = None,
        original_txn_currency: str = None,
        pos_entry_mode: SimulateCardTransactionAuthorizationRequestEeaPosEntryMode = None,
        retrieval_reference_number: str = None,
        systems_trace_audit_number: str = None,
        wallet_transaction_id: str = None,
    ):
        """SimulateCardTransactionAuthorizationRequestEea

        :param amount: Amount of the transaction, in units defined in `currency`., defaults to None
        :type amount: float, optional
        :param auth_response: Response related to an authorization. Contains the following fields:, defaults to None
        :type auth_response: SimulateCardTransactionAuthorizationRequestEeaAuthResponse, optional
        :param card_id: ID of the card. String starting with **card_**., defaults to None
        :type card_id: str, optional
        :param card_transaction_id: ID of the transaction. String starting with **cit_**., defaults to None
        :type card_transaction_id: str, optional
        :param currency_code: Currency of the transaction. Three-letter ISO 4217 code.Currency of the refund transaction. Three-letter ISO 4217 code., defaults to None
        :type currency_code: str, optional
        :param fx_rate: The exchange rate. Relevant to capture (clearing) transactions., defaults to None
        :type fx_rate: str, optional
        :param id_: ID of the issued card transaction. String starting with **cit_**., defaults to None
        :type id_: str, optional
        :param issuing_txn_type: Type of transaction on the issued card., defaults to None
        :type issuing_txn_type: IssuingTxnType, optional
        :param last4: Last 4 digits of the card number., defaults to None
        :type last4: str, optional
        :param merchant_category_code: Four-digit merchant category code (MCC) of the initiator of the transaction, as defined in ISO 18245., defaults to None
        :type merchant_category_code: str, optional
        :param merchant_name_location: Name and location of the merchant., defaults to None
        :type merchant_name_location: str, optional
        :param original_transaction_id: ID of the original issued card transaction. String starting with **cit_**., defaults to None
        :type original_transaction_id: str, optional
        :param original_txn_amount: Original amount for FX transactions, when `currency` is different from `original_txn_currency`., defaults to None
        :type original_txn_amount: float, optional
        :param original_txn_currency: Original currency in an FX transaction., defaults to None
        :type original_txn_currency: str, optional
        :param pos_entry_mode: The mode of entry of the transaction at the point of sale., defaults to None
        :type pos_entry_mode: SimulateCardTransactionAuthorizationRequestEeaPosEntryMode, optional
        :param retrieval_reference_number: Retrieval reference number for the card transaction., defaults to None
        :type retrieval_reference_number: str, optional
        :param systems_trace_audit_number: Reserved., defaults to None
        :type systems_trace_audit_number: str, optional
        :param wallet_transaction_id: ID of the wallet transaction String starting with **wt_**., defaults to None
        :type wallet_transaction_id: str, optional
        """
        self.amount = self._define_number("amount", amount, nullable=True)
        self.auth_response = self._define_object(
            auth_response, SimulateCardTransactionAuthorizationRequestEeaAuthResponse
        )
        self.card_id = self._define_str("card_id", card_id, nullable=True)
        self.card_transaction_id = self._define_str(
            "card_transaction_id", card_transaction_id, nullable=True
        )
        self.currency_code = self._define_str(
            "currency_code", currency_code, nullable=True
        )
        self.fx_rate = self._define_str("fx_rate", fx_rate, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.issuing_txn_type = (
            self._enum_matching(
                issuing_txn_type, IssuingTxnType.list(), "issuing_txn_type"
            )
            if issuing_txn_type
            else None
        )
        self.last4 = self._define_str("last4", last4, nullable=True)
        self.merchant_category_code = self._define_str(
            "merchant_category_code", merchant_category_code, nullable=True
        )
        self.merchant_name_location = self._define_str(
            "merchant_name_location", merchant_name_location, nullable=True
        )
        self.original_transaction_id = self._define_str(
            "original_transaction_id", original_transaction_id, nullable=True
        )
        self.original_txn_amount = self._define_number(
            "original_txn_amount", original_txn_amount, nullable=True
        )
        self.original_txn_currency = self._define_str(
            "original_txn_currency", original_txn_currency, nullable=True
        )
        self.pos_entry_mode = (
            self._enum_matching(
                pos_entry_mode,
                SimulateCardTransactionAuthorizationRequestEeaPosEntryMode.list(),
                "pos_entry_mode",
            )
            if pos_entry_mode
            else None
        )
        self.retrieval_reference_number = self._define_str(
            "retrieval_reference_number", retrieval_reference_number, nullable=True
        )
        self.systems_trace_audit_number = self._define_str(
            "systems_trace_audit_number", systems_trace_audit_number, nullable=True
        )
        self.wallet_transaction_id = self._define_str(
            "wallet_transaction_id", wallet_transaction_id, nullable=True
        )
